@@ -307,10 +307,11 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
     return PyErr_Occurred() ? 0 : 1;
 }
 
+/* copy row of coordinate array from location at _p to _coor */
 #define CASE_MAP_COORDINATES(_p, _coor, _rank, _stride, _type) \
 case t ## _type:                                                    \
 {                                                              \
-    int _hh;                                                     \
+    npy_intp _hh;                                               \
     for(_hh = 0; _hh < _rank; _hh++) {                           \
         _coor[_hh] = *(_type*)_p;                                  \
         _p += _stride;                                             \
@@ -353,7 +354,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
     char *po, *pi, *pc = NULL;
     npy_intp **edge_offsets = NULL, **data_offsets = NULL, filter_size;
     npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
-    npy_intp cstride = 0, kk, hh, ll, jj, *idxs = NULL;
+    npy_intp cstride = 0, kk, hh, ll, jj;
     npy_intp size;
     double **splvals = NULL, icoor[MAXDIM];
     npy_intp idimensions[MAXDIM], istrides[MAXDIM];
@@ -415,11 +416,6 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
     filter_size = 1;
     for(jj = 0; jj < irank; jj++)
         filter_size *= order + 1;
-    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
-    if (!idxs) {
-        PyErr_NoMemory();
-        goto exit;
-    }
 
     /* initialize output iterator: */
     if (!NI_InitPointIterator(output, &io))
@@ -482,7 +478,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         } else if (coordinates) {
             /* mapping is from an coordinates array: */
             char *p = pc;
-            switch(coordinates->descr->type_num) {
+            switch (NI_NormalizeType(coordinates->descr->type_num)) {
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, Bool);
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, UInt8);
                 CASE_MAP_COORDINATES(p, icoor, irank, cstride, UInt16);
@@ -553,9 +549,13 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
 
         if (!constant) {
             npy_intp *ff = fcoordinates;
+            const int type_num = NI_NormalizeType(input->descr->type_num);
+            t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
+                double coeff = 0.0;
                 int idx = 0;
-                if (edge) {
+
+                if (NI_UNLIKELY(edge)) {
                     for(ll = 0; ll < irank; ll++) {
                         if (edge_offsets[ll])
                             idx += edge_offsets[ll][ff[ll]];
@@ -566,29 +566,20 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                     idx = foffsets[hh];
                 }
                 idx += offset;
-                idxs[hh] = idx;
-                ff += irank;
-            }
-        }
-        if (!constant) {
-            npy_intp *ff = fcoordinates;
-            t = 0.0;
-            for(hh = 0; hh < filter_size; hh++) {
-                double coeff = 0.0;
-                switch(input->descr->type_num) {
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt32);
+                switch (type_num) {
+                    CASE_INTERP_COEFF(coeff, pi, idx, Bool);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt32);
 #if HAS_UINT64
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt64);
 #endif
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int64);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float64);
                 default:
                     PyErr_SetString(PyExc_RuntimeError,
                                                     "data type not supported");
@@ -605,14 +596,17 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
             t = cval;
         }
         /* store output value: */
-        switch (output->descr->type_num) {
+        switch (NI_NormalizeType(output->descr->type_num)) {
             CASE_INTERP_OUT(po, t, Bool);
             CASE_INTERP_OUT_UINT(po, t, UInt8, 0, MAX_UINT8);
             CASE_INTERP_OUT_UINT(po, t, UInt16, 0, MAX_UINT16);
             CASE_INTERP_OUT_UINT(po, t, UInt32, 0, MAX_UINT32);
 #if HAS_UINT64
-            /* FIXME */
-            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, MAX_UINT32);
+            /* There was a bug in numpy as of (at least) <= 1.6.1 such that
+             * MAX_UINT64 was incorrectly defined, leading to a compiler error.
+             * NPY_MAX_UINT64 is correctly defined
+             */
+            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, NPY_MAX_UINT64);
 #endif
             CASE_INTERP_OUT_INT(po, t, Int8, MIN_INT8, MAX_INT8);
             CASE_INTERP_OUT_INT(po, t, Int16, MIN_INT16, MAX_INT16);
@@ -648,8 +642,6 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         free(foffsets);
     if (fcoordinates)
         free(fcoordinates);
-    if (idxs)
-        free(idxs);
     return PyErr_Occurred() ? 0 : 1;
 }
 
@@ -661,7 +653,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     npy_intp **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
     npy_intp ftmp[MAXDIM], *fcoordinates = NULL, *foffsets = NULL;
     npy_intp jj, hh, kk, filter_size, odimensions[MAXDIM];
-    npy_intp idimensions[MAXDIM], istrides[MAXDIM], *idxs = NULL;
+    npy_intp idimensions[MAXDIM], istrides[MAXDIM];
     npy_intp size;
     double ***splvals = NULL;
     NI_Iterator io;
@@ -789,11 +781,6 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     filter_size = 1;
     for(jj = 0; jj < rank; jj++)
         filter_size *= order + 1;
-    idxs = (npy_intp*)malloc(filter_size * sizeof(idxs));
-    if (!idxs) {
-        PyErr_NoMemory();
-        goto exit;
-    }
 
     if (!NI_InitPointIterator(output, &io))
         goto exit;
@@ -847,9 +834,13 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
 
         if (!zero) {
             npy_intp *ff = fcoordinates;
+            const int type_num = NI_NormalizeType(input->descr->type_num);
+            t = 0.0;
             for(hh = 0; hh < filter_size; hh++) {
                 int idx = 0;
-                if (edge) {
+                double coeff = 0.0;
+
+                if (NI_UNLIKELY(edge)) {
                         /* use precalculated edge offsets: */
                     for(jj = 0; jj < rank; jj++) {
                         if (edge_offsets[jj][io.coordinates[jj]])
@@ -862,29 +853,20 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                     /* use normal offsets: */
                     idx += oo + foffsets[hh];
                 }
-                idxs[hh] = idx;
-                ff += rank;
-            }
-        }
-        if (!zero) {
-            npy_intp *ff = fcoordinates;
-            t = 0.0;
-            for(hh = 0; hh < filter_size; hh++) {
-                double coeff = 0.0;
-                switch(input->descr->type_num) {
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Bool);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt32);
+                switch (type_num) {
+                    CASE_INTERP_COEFF(coeff, pi, idx, Bool);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt32);
 #if HAS_UINT64
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], UInt64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, UInt64);
 #endif
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int8);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int16);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Int64);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float32);
-                    CASE_INTERP_COEFF(coeff, pi, idxs[hh], Float64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int8);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int16);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Int64);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float32);
+                    CASE_INTERP_COEFF(coeff, pi, idx, Float64);
                 default:
                     PyErr_SetString(PyExc_RuntimeError,
                                                     "data type not supported");
@@ -901,14 +883,17 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
             t = cval;
         }
         /* store output: */
-        switch (output->descr->type_num) {
+        switch (NI_NormalizeType(output->descr->type_num)) {
             CASE_INTERP_OUT(po, t, Bool);
             CASE_INTERP_OUT_UINT(po, t, UInt8, 0, MAX_UINT8);
             CASE_INTERP_OUT_UINT(po, t, UInt16, 0, MAX_UINT16);
             CASE_INTERP_OUT_UINT(po, t, UInt32, 0, MAX_UINT32);
 #if HAS_UINT64
-            /* FIXME */
-            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, MAX_UINT32);
+            /* There was a bug in numpy as of (at least) <= 1.6.1 such that
+             * MAX_UINT64 was incorrectly defined, leading to a compiler error.
+             * NPY_MAX_UINT64 is correctly defined
+             */
+            CASE_INTERP_OUT_UINT(po, t, UInt64, 0, NPY_MAX_UINT64);
 #endif
             CASE_INTERP_OUT_INT(po, t, Int8, MIN_INT8, MAX_INT8);
             CASE_INTERP_OUT_INT(po, t, Int16, MIN_INT16, MAX_INT16);
@@ -962,7 +947,5 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         free(foffsets);
     if (fcoordinates)
         free(fcoordinates);
-    if (idxs)
-        free(idxs);
     return PyErr_Occurred() ? 0 : 1;
 }
